local addon, _ = ...

local defaults = {
   ["version"] = "1.0",
   ["frame"] = {
	  ["scale"] = 1.2,
	  ["lock"] = 0,
	  ["horziontal"] = false,
   },
   ["button"] = {
	  -- the alpha of button status
	  ["show"] = 1,
	  ["hide"] = 0.25,
   },
   -- buffs = { [spellID] = 1 }
   ["buffs"] = {
	  [41450] = 1,
   },
}

local allBtns = {}; 

-- init function defination

function InitMover(self)
   local mover = CreateFrame("frame", "CancelAuraSMover", self);
   mover:SetSize(70, 20);
   mover:SetBackdrop({
						bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
						tile = true,
					 });
   mover:SetPoint("BOTTOM", self, "TOP",0,7);
   -- create text on mover
   local moverText = mover:CreateFontString(nil, "OVERLAY", "GAMEFONTNORMAL");
   moverText:SetText("drag me!");
   moverText:SetTextColor(1,1,1,1);
   moverText:SetPoint("CENTER", mover, "CENTER");
   -- 
   mover.text = moverText;

   mover:SetScript("OnMouseDown",
				  function(self,button)
					 if (button=="LeftButton") then
						self:GetParent():StartMoving();
					 end
				  end);
   
   mover:SetScript("OnMouseUp",
				  function(self)
					 self:GetParent():StopMovingOrSizing();
				  end);
   
end	

function InitAllButtons(buffs)
   -- buffs = { [spellID] = 1 }
   local buttons = {};
   local name, icon, _; 
   local i=1;
   local lastBtn = false;
   local button;
    
   for k,v in pairs(buffs) do
	  name, _, icon = GetSpellInfo(k);
	  button = _G["CancelAuraSButton"..i];
	  if not button then
		 button = CreateFrame("button", "CancelAuraSButton"..i, CancelAuraSFrame, "CancelAuraSButtonTemplate");
	  end
	  button:SetNormalTexture(icon);
	  button:SetAttribute("macrotext", "/cancelaura "..name);
	  
	  button:ClearAllPoints();
	  if (lastBtn) then
		 if (CancelAuraS.frame.horziontal) then
			button:SetPoint("LEFT", lastBtn, "RIGHT", 7, 0);
		 else
			button:SetPoint("TOP", lastBtn, "BOTTOM", 0, -7);
		 end
	  else
		 button:SetPoint("TOPLEFT", CancelAuraSFrame, "TOPLEFT");
	  end
	  button:Show();
	  
	  lastBtn = button;
	  buttons[k] = button;
	  i = i + 1;
   end
   
   -- hide other btns
   button = _G["CancelAuraSButton"..i];
   while (button) do
	  button:Hide();
	  i = i + 1;
	  button = _G["CancelAuraSButton"..i];
   end
   
   return buttons;
end

function InitFrame()
   local frame = CreateFrame("frame", "CancelAuraSFrame", UIParent);
   frame:SetWidth(64);
   frame:SetHeight(192);
   frame:SetPoint("LEFT", UIParent, "CENTER"); 
   frame:SetMovable(true);
   
   InitMover(frame);
   
   frame:SetScript("OnEvent", function(self, event, ...)
					  if (type(self[event]) == 'function') then
								 self[event](self, ...);
					  end
							  end);
   frame:RegisterEvent("PLAYER_LOGIN");
   frame:RegisterEvent("UNIT_AURA");
   
   function frame:PLAYER_LOGIN(addon)
	  if not CancelAuraS then
		 CancelAuraS = copyDefaults(defaults);
	  elseif (CancelAuraS.version ~= defaults.version) then
		 CancelAuraS = copyDefaults(defaults);
		 print("CancelAuraS updated to ".. defaults.version);
	  end
	  
	  checkAndUpdateBtns()
	  
	  CancelAuraSFrame:SetScale(CancelAuraS.frame.scale);
	  if (CancelAuraS.frame.lock > 0) then
		 CancelAuraSMover:Hide();
	  end
	  
   end
   
   function frame:PLAYER_REGEN_ENABLED() 
	  allBtns = InitAllButtons(CancelAuraS.buffs);
	  updateBuffs(CancelAuraS.buffs);
	  self:UnregisterEvent("PLAYER_REGEN_ENABLED");
   end
   
   function frame:UNIT_AURA(unit)
	  if (unit == "player") then
		 updateBuffs(CancelAuraS.buffs);
	  end
   end
end

function updateBuffs(buffs)
   -- buffs = { [spellID] = 1 };
   local i = 1;
   for k, v in pairs(allBtns) do
	  v:SetAlpha(CancelAuraS.button.hide);
   end
   
   local spellID = select(11, UnitBuff("player", i));
   while(spellID) do
	  if (buffs[spellID]) then
		 allBtns[spellID]:SetAlpha(CancelAuraS.button.show);
	  end
	  i = i + 1;
	  spellID = select(11, UnitBuff("player", i));
   end
end

InitFrame();

-- tool functions --

function checkAndUpdateBtns()
   if (InCombatLockdown()) then
	  CancelAuraSFrame:RegisterEvent("PLAYER_REGEN_ENABLED");
   else
	  allBtns = InitAllButtons(CancelAuraS.buffs);
	  updateBuffs(CancelAuraS.buffs);
   end
end

-- This function copies values from one table into another:
function copyDefaults(src, dst)
   -- If no source (defaults) is specified, return an empty table:
   if type(src) ~= "table" then return {} end
   -- If no target (saved variable) is specified, create a new table:
   if type(dst) then dst = {} end
   -- Loop through the source (defaults):
   for k, v in pairs(src) do
	  -- If the value is a sub-table:
	  if type(v) == "table" then
		 -- Recursively call the function:
		 dst[k] = copyDefaults(v, dst[k])
		 -- Or if the default value type doesn't match the existing value type:
	  elseif type(v) ~= type(dst[k]) then
		 -- Overwrite the existing value with the default one:
		 dst[k] = v
	  end
   end
   -- Return the destination table:
   return dst
end

-- slash commands --

SLASH_CANCELAURAS1 = "/cancelauras"
SLASH_CANCELAURAS2 = "/cas"

SlashCmdList["CANCELAURAS"] = function(args)
   local command, arg1 = strsplit(" ", args, 2);
   command = command:lower();
   local frame = CancelAuraSFrame;
   local mover = CancelAuraSMover;
   if (command == "hide") then
	  frame:Hide();
   elseif (command == "unlock") then
	  CancelAuraS.frame.lock = 0;
	  mover:Show();
	  frame:Show();
   elseif (command == "lock") then
	  CancelAuraS.frame.lock = 1;
	  mover:Hide();
   elseif (command == "scale") then
	  local size = tonumber(arg1);
	  if (type(size) == 'number' and size > 0) then
		 frame:SetScale(size);
		 CancelAuraS.frame.scale = size;
	  end
   elseif (command == "list") then
	  local count = 0;
	  for v in pairs(CancelAuraS.buffs) do
		 print(v .. " - " .. GetSpellLink(v));
		 count = 1;
	  end
	  if (count == 0) then
		 print("no spellID in list.");
	  end
   elseif (command == "add") then
	  local spellID = tonumber(arg1);
	  if (type(spellID) == 'number') then
		 CancelAuraS.buffs[spellID] = 1;
		 print("|cff00ff00Add|r "..GetSpellLink(spellID).." |cff00ff00into list.|r");
		 checkAndUpdateBtns();
	  end
   elseif (command == "remove" or command == "rm" ) then
	  local spellID = tonumber(arg1);
	  if (type(spellID) == 'number') then
		 CancelAuraS.buffs[spellID] = nil;
		 print("|cffff3300Remove|r "..GetSpellLink(spellID).." |cffff3300from list.|r");
		 checkAndUpdateBtns();
	  end
   elseif (command == "vh") then
	  CancelAuraS.frame.horziontal = not CancelAuraS.frame.horziontal;
	  checkAndUpdateBtns();
   else
	  cmdInfo()
   end
   
end
									   
function cmdInfo()
   print([[
|cff33ccffC|rancel|cff33ccffA|rura|cff33ccffS|r Usage:
/|cff33ccffcas|r lock||unlock
/|cff33ccffcas|r scale
/|cff33ccffcas|r |cff00ff00add|r|||cffff3300rm|r spellID
/|cff33ccffcas|r list       |cfffff000list the current watching auras|r
/|cff33ccffcas|r vh      |cfffff000Toggle between Vertical and Horziontal|r
]]);
end

-- bindings setting
--BINDING_HEADER_CANCELAURAS = "CancelAuraS";
--BINDING_NAME_CANCELAURASBTN1 = "Cancel Aura 1";
